@@ -1,13 +1,15 @@
 /*
  * device backend utilities
  *
- * Copyright (C) 2004, Christophe Saout <christophe@saout.de>
+ * Copyright (C) 2004, Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007, Clemens Fruhwirth <clemens@endorphin.org>
  * Copyright (C) 2009-2012, Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2012, Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,25 +46,35 @@ static int device_ready(const char *device)
 	int devfd, r = 0;
 	struct stat st;
 
-	if(stat(device, &st) < 0) {
+	log_dbg("Trying to open and read device %s.", device);
+	devfd = open(device, O_RDONLY);
+	if (devfd < 0) {
 		log_err(NULL, _("Device %s doesn't exist or access denied.\n"), device);
 		return -EINVAL;
 	}
-
-	if (!S_ISBLK(st.st_mode))
-		return S_ISREG(st.st_mode) ? -ENOTBLK : -EINVAL;
-
-	log_dbg("Trying to open and read device %s.", device);
-	devfd = open(device, O_RDONLY | O_DIRECT | O_SYNC);
-	if(devfd < 0) {
-		log_err(NULL, _("Cannot open device %s.\n"), device);
-		return -EINVAL;
-	}
+	if (fstat(devfd, &st) < 0)
+		r = -EINVAL;
+	else if (!S_ISBLK(st.st_mode))
+		r = S_ISREG(st.st_mode) ? -ENOTBLK : -EINVAL;
 
 	close(devfd);
 	return r;
 }
 
+int device_open(struct device *device, int flags)
+{
+	int devfd;
+
+	devfd = open(device_path(device), flags | O_DIRECT | O_SYNC);
+	if (devfd < 0 && errno == EINVAL) {
+		log_dbg("Trying to open device %s without direct-io.",
+			device_path(device));
+		devfd = open(device_path(device), flags | O_SYNC);
+	}
+
+	return devfd;
+}
+
 int device_alloc(struct device **device, const char *path)
 {
 	struct device *dev;
@@ -209,7 +221,7 @@ int device_block_size(struct device *device)
 	if (fstat(fd, &st) < 0)
 		goto out;
 
-	if (S_ISREG(st.st_mode)) {
+	if (S_ISREG(st.st_mode) || device->file_path) {
 		r = (int)crypt_getpagesize();
 		goto out;
 	}
@@ -282,6 +294,7 @@ static int device_info(struct device *device,
 		flags |= O_EXCL;
 
 	/* Try to open read-write to check whether it is a read-only device */
+	/* coverity[toctou] */
 	fd = open(device->path, O_RDWR | flags);
 	if (fd == -1 && errno == EROFS) {
 		*readonly = 1;
